(* This file defines an interpreter for our small programming language *)

open Printf
open AbstractSyntax

(* This exception is raised when the interpreter encounters a fatal error *)
exception EvaluationError of string

(* Finite maps from value_variables to some data type *)
module Env =
  Map.Make(struct type t = value_variable let compare = compare end)

(* The representation of values (results of evaluation) *)
type value =
  | Con of data_constructor * value list
  (* | ??? *)

(* Environments are finite maps from value_variables to values *)
and environment = value Env.t

(* Return the value associated with [var] in [env]. *)
let find_var var env =
  try Env.find var env
  with Not_found -> raise (EvaluationError ("unbound variable " ^ var))

(* Return an environment that extends [env] with a binding from 
   [var] to [value] *)
let bind_var var value env =
  Env.add var value env

(* Print a value on standard output *)
let rec print_value v =
  match v with
  | Con(cstr, []) ->
      printf "%s" cstr
  | Con(cstr, v1 :: vl) ->
      printf "%s(" cstr;
      print_value v1;
      List.iter (fun v -> printf ", "; print_value v) vl;
      printf ")"
  (* | ??? *)

(* [eval env expr] returns the value of expression [expr] in environment [env] *)
let rec eval env expr =
  raise (EvaluationError "to be implemented")

(* [eval_match env cstr args branches] evaluates a pattern-matching.
   The value being matched is [Con(cstr, args)].  *)

and eval_match env cstr args branches =
  raise (EvaluationError "to be implemented")

(* Evaluate the given program and print its value. *)
let eval_program (Program(datatypes, body)) =
  try
    print_value (eval Env.empty body); print_newline()
  with EvaluationError msg ->
    printf "Error during evaluation: %s\n" msg
